#!/usr/bin/env perl
#
#
# The fixer is needed to change the smt2 file generated by hycomp to be readable by z3.
# z3 complains about integers number inside the to_real construct, even if they are constants.
# This script just processes the smt file and replaces (to_real int_number) with (to_real real_number)
#
# For example, (to_real 1) becomes (to_real 1.0)
#

use strict;
use warnings;

my $Z3PATH="z3";

sub usage() {
    printf "usage: perl z3fixer.pl \n";
    printf "options:\n";
    printf "\t -h show help\n";
    printf "\t -smt2 <file>\n";
    exit 1;
}

my $infile="";
my $argno=0;
my $app=$#ARGV;
while ($argno <= $app) {
    if ($ARGV[$argno] eq "-h" || $ARGV[$argno] eq "--help") {
        $argno++;
        usage;
    }
    elsif ($ARGV[$argno] eq "-smt2") {
        $argno++;

        if ($ARGV[$argno] ne "") {
            $infile = $ARGV[$argno];
        }
        else {
            usage;
        }
        $argno++;
    }
    else {
        $argno++;                
    }
}

open FILE, $infile or die $!;
my $newsmt2="";
my $define="";
while(<FILE>) {
    chomp();
    my $line = $_;

    # Substitutes - with ~
    while($line =~ m/^(.*?)\(- (\d+)\)(.*)$/) {
	$line = "$1(~ $2)$3";
    }

    # Substitutes - ( with ~ (
    while($line =~ m/^(.*?)\(- \((.*)$/) {
	$line = "$1(~ ($2";
    }

    # substiute (/ n  with (/ n.0 
    while($line =~ m/^(.*?)\(\/ (\d+) (.*)$/) {
	$line = "$1(/ $2.0 $3";
    }

    # substiute (/ n.n n) with (/ n.n n.0)
    while($line =~ m/^(.*?)\(\/ (\d+\.\d+) (\d+)\)(.*)$/) {
	$line = "$1(/ $2 $3.0)$4";
    }

    # keep track of defines 
    if($line =~ m/^(.*?)declare-fun(.*)$/) {
        $define="${define}\n${line}";
    }

    # Substitutes (reset-assertions) wit (reset)
    if($line =~ m/^(.*?)\(reset-assertions\)(.*)$/) {
        $line = "$1(reset)$2\n${define}";
    }

    # Substitutes (to_real [0-9]+) wit [0-9]+.0
    if($line =~ m/^(.*?)\(to_real (\d+)\)(.*)$/) {
        $line = "$1 $2.0 $3";
    }
    # Substitutes (to_real (~[0-9]+)) wit [0-9]+.0
    if($line =~ m/^(.*?)\(to_real \(~ (\d+)\)\)(.*)$/) {
        $line = "$1 (~ $2.0) $3";
    }

    $newsmt2 = $newsmt2 . "$line\n";
}
close FILE;

open FILE, ">$infile" or die $!;
print FILE $newsmt2;
print FILE "\n";
close FILE;


my $z3out=`$Z3PATH -smt2 $infile`;
print $z3out;
